/******************************************************************************
 * shaderProgram.cpp
 * implements the base class for GLSL shaders
 * Copyright (C) 2010 Marc Gissler <gisslerm@informatik.uni-freiburg.de>
 ******************************************************************************/

#include "shaderProgram.h"

CShaderProgram::CShaderProgram()
{
}

CShaderProgram::~CShaderProgram()
{
}

bool CShaderProgram::isValid()
{
	if(!glIsShader(m_vertexShader))
		return false;
	if(!glIsShader(m_fragmentShader))
		return false;
	if(!glIsProgram(m_shaderProgram))
		return false;

	return true;
}

bool CShaderProgram::m_compileShader(GLuint &shader, GLenum shadertype, const GLchar *source)
{
	shader = glCreateShader(shadertype);
	glShaderSource(shader, 1, &source, NULL);
	glCompileShader(shader);
	return m_checkValidity(shader);
}

bool CShaderProgram::m_checkValidity(GLuint shader)
{
	int shadererror;
	int infologLength = 0;
	int charsWritten  = 0;
	char *infoLog;

	glGetShaderiv(shader, GL_COMPILE_STATUS, &shadererror);
	if (shadererror==GL_TRUE) return true;

	glGetShaderiv(shader, GL_INFO_LOG_LENGTH,&infologLength);

	if (infologLength>0)
	{
		infoLog=new char[infologLength];
		glGetShaderInfoLog(shader, infologLength, &charsWritten, infoLog);

		GLchar* infoLogText = new GLchar[infologLength];
		for(int i = 0; i <= infologLength; i++)
		{
			infoLogText[i]= infoLog[i];
		}		
		printf(infoLogText);
		delete(infoLog);
	}
	return false;
}

bool CShaderProgram::m_buildShaderProgram()
{
	m_shaderProgram = glCreateProgram();

	glAttachShader(m_shaderProgram, m_vertexShader);
	glAttachShader(m_shaderProgram, m_fragmentShader);
	
	glLinkProgram(m_shaderProgram);

	int shadererror;
	glGetProgramiv(m_shaderProgram, GL_LINK_STATUS, &shadererror);
	
	if (shadererror == GL_FALSE)
	{
		GLchar* infoLog = new GLchar[500];
		GLsizei length;
		glGetProgramInfoLog(m_shaderProgram, 500, &length, infoLog);

		glDetachShader(m_shaderProgram, m_vertexShader);
		glDetachShader(m_shaderProgram, m_fragmentShader);
		glDeleteShader(m_vertexShader);
		glDeleteShader(m_fragmentShader);
		glDeleteProgram(m_shaderProgram);
		GLString errmsg = "Cannot link shader.";
		printf(errmsg.c_str());
		return false;
	}

	return true;
}
