/******************************************************************************
 * shadow map shader code
 * provides vertex and fragment shader in the form of a character string
 * Copyright (C) 2010 Marc Gissler <gisslerm@informatik.uni-freiburg.de>
 ******************************************************************************/
#pragma once

#include "globals.h"

const GLchar *shader_shadowMap_vert =
"varying vec4 ShadowCoord;\n"
"\n"
"void main(void)\n"
"{\n"
"	// mixed\n"
"	gl_Position = ftransform();\n"
"	gl_FrontColor = gl_Color;\n"
"	\n"
"	ShadowCoord = gl_TextureMatrix[7] * gl_TextureMatrix[6] * gl_ModelViewMatrix * gl_Vertex;\n"
"}\n";

const GLchar *shader_shadowMap_frag =
"uniform sampler2D shadowMap;\n"
"\n"
"uniform vec4  shadowColor;\n"
"uniform float shadowFactor;\n"
"\n"
"varying vec4 ShadowCoord;\n"
"\n"
"\n"
"void main()\n"
"{\n"
"	vec4 color = vec4(0.0, 0.0, 0.0, 0.0);\n"
"	\n"
"	vec4 ShadowCoordPostW;		\n"
"	ShadowCoordPostW = ShadowCoord / ShadowCoord.w;\n"
"	\n"
"	//ShadowCoordPostW = ShadowCoordPostW * 0.5 + 0.5;\n"
"	\n"
"	vec2 moments = texture2D(shadowMap, ShadowCoordPostW.xy).rg;\n"
"	\n"
"	float percentNotLit = 0.0;\n"
"	\n"
"	// Surface is fully lit. as the current fragment is before the light occluder\n"
"	if (ShadowCoordPostW.z <= moments.x - 0.0001)\n" //0.00001
"	{\n"
"		percentNotLit = 0.0;\n"
"	}\n"
"	else\n"
"	{\n"
"		float variance = moments.y - (moments.x * moments.x);\n"
"		\n"
"		variance = max( variance, 0.002);\n" //0.00002
"		float d = ShadowCoordPostW.z - moments.x - 0.00001;\n"
"		percentNotLit = variance / (variance + d*d);\n"
"	}\n"
"	\n"
"	//percentNotLit = clamp( (percentNotLit  0.1) / (1.0  0.1), 0.0, 1.0);\n"
"	\n"
"	color = shadowColor;\n"
"	//color.a = percentNotLit * shadowFactor;\n" // option 1
"  color.a = (percentNotLit > 0.001) ? 1.0 : 0.0\n;" // option2
"	\n"
"	gl_FragColor = clamp(color, 0.0, 1.0);\n"
"}\n";
